
import torch
import torch.nn as nn

class SiameseNetwork(nn.Module):

    def __init__(self):
        super(SiameseNetwork, self).__init__()

        # Setting up the Sequential of CNN Layers
        self.cnn1 = nn.Sequential(
            nn.Conv2d(3, 96, kernel_size=11,stride=4),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(3, stride=2),
            
            nn.Conv2d(96, 256, kernel_size=5, stride=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, stride=2),

            nn.Conv2d(256, 384, kernel_size=3,stride=1),
            nn.ReLU(inplace=True)
        )

        # Setting up the Fully Connected Layers
        self.fc1 = nn.Sequential(
            nn.Linear(384, 1024),
            nn.ReLU(inplace=True),
            
            nn.Linear(1024, 256),
            nn.ReLU(inplace=True),
            
            nn.Linear(256,2)
        )
        
    def forward_once(self, x):
        # This function will be called for both images
        # It's output is used to determine the similiarity
        output = self.cnn1(x)
        output = output.view(output.size()[0], -1)
        output = self.fc1(output)
        return output

    def forward(self, input1, input2):
        # In this function we pass in both images and obtain both vectors
        # which are returned
        output1 = self.forward_once(input1)
        output2 = self.forward_once(input2)

        return output1, output2




import torch
from .model import SiameseNetwork
from PIL import Image
import PIL.ImageOps
import torchvision.transforms as transforms
import torch.nn.functional as F

def videoBarcodeSimilarities(image1_path: str, image2_path: str):
    model = SiameseNetwork()
    PATH = "./dropdart.h5"
    # weights = torch.hub.load_state_dict_from_url(
    #     "https://github.com/smilingprogrammer/RealTimeNudityDetectionAlgorithm/blob/main/barimgsim.pt", progress=True
    # )
    model.load_state_dict(torch.load(PATH))
    model.eval()

    image1 = Image.open(image1_path)
    image2 = Image.open(image2_path)

    # Preprocess the images for ResNet50
    preprocess = transforms.Compose([
    transforms.Resize((100, 100)),
    transforms.ToTensor()
    ])

    image1 = preprocess(image1)
    image2 = preprocess(image2)

    # Add batch dimension to images
    image1 = image1.unsqueeze(0)
    image2 = image2.unsqueeze(0)
    output1, output2 = model(image1, image2)

    euclidean_distance = F.pairwise_distance(output1, output2)
    max_distance = torch.sqrt(torch.tensor(output1.shape[1], dtype=torch.float))
    similarity_score = 1.0 / (1.0 + euclidean_distance / max_distance)
    print(f'Similarity score between barImage1 and barImage2: {similarity_score.item()}')



from setuptools import setup, find_packages

setup(
    name="VidBarcodeSimilarities",
    version="0.0.1",
    description="A library to compare similarities between barcode images",
    author="Abdulsobur",
    author_email="Abdulsoburoyewale@gmail.com",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
